# Dumps metadata about local Windows event logs
# For diagnositc purposes

param(
    [Parameter(Mandatory = $true)]
    [string[]] $LogNames
)

Set-StrictMode -Version Latest

function NewAPIGetFirstEvents($logName)
{
    wevtutil qe $logName /q:"Event/*[EventRecordID >= 0]" /c:10 /f:Text
}

function NewAPIGetLastEventsQuery($logName, $finalRn)
{
    wevtutil qe $logName /q:"Event/*[EventRecordID >= $($finalRn - 9)]" /c:10 /f:Text
}

function NewAPIGetLastEventsSeek($logName, $finalRn)
{
    $bookmarkFile = [io.path]::GetTempFileName()
    "<BookmarkList><Bookmark Channel='$logName' RecordId='$($finalRn - 10)' IsCurrent = 'true' /></BookmarkList>" | Out-File $bookmarkFile
    wevtutil qe $logName /bm:$bookmarkFile /c:10 /f:Text
}

function OldAPIGetFirstEvents($logName)
{
    $log = new-object System.Diagnostics.EventLog $logName
    $e = $log.Entries.GetEnumerator()
    for($i = 0; ($i -lt 10) -and $e.MoveNext(); $i++)
    {
        $e.Current
    }
}

function OldAPIGetLastEvents($logName)
{
    $log = new-object System.Diagnostics.EventLog $logName
    $minRn = [math]::Max(0, $log.Entries.Count - 10)
    $maxRn = [math]::Max(0, $log.Entries.Count - 1)

    $minRn .. $maxRn |%{ $log.Entries[$_] }
}

$isAdmin = ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")
if(-not $isAdmin)
{
    Write-Host "This script should be run as an administrator" -fore Yellow -back Black 
    exit
}

Get-WinEvent -ListLog $LogNames |?{ $_.RecordCount -gt 0 } |%{
    $log = $_

    $lastRecord = ($log | Get-WinEvent -MaxEvents 1).RecordId
    $firstRecord = ($log | Get-WinEvent -MaxEvents 1 -Oldest).RecordId

    $newApiFirstEventsTime = Measure-Command { NewAPIGetFirstEvents $log.LogName }
    $newApiLastEventsTimeQuery = Measure-Command { NewAPIGetLastEventsQuery $log.LogName $lastRecord }
    $newApiLastEventsTimeSeek = Measure-Command { NewAPIGetLastEventsSeek $log.LogName $lastRecord }

    $oldApiFirstEventsTime =
        if($log.IsClassicLog) { Measure-Command { OldAPIGetFirstEvents $log.LogName } | select -expand TotalSeconds }
        else { 'n/a' }
    $oldApiLastEventsTime =
        if($log.IsClassicLog) { Measure-Command { OldAPIGetLastEvents $log.LogName $lastRecord } | select -expand TotalSeconds }
        else { 'n/a' }

    New-Object PsCustomObject -Property @{
        'Log' = $log.LogName
        'RecCount' = $log.RecordCount
        'Providers' = $log.ProviderNames.Count
        'FirstRN' = $firstRecord
        'LastRN' = $lastRecord
        'NewFirstTime' = $newApiFirstEventsTime.TotalSeconds
        'NewLastTimeQ' = $newApiLastEventsTimeQuery.TotalSeconds
        'NewLastTimeS' = $newApiLastEventsTimeSeek.TotalSeconds
        'OldFirstTime' = $oldApiFirstEventsTime
        'OldLastTime' = $oldApiLastEventsTime
    }
}