Param(
	[string[]] $computernames=@(),
	[system.management.automation.PSCredential] $credential = $null
)

$currentComputerName = "$env:computername.$env:userdnsdomain"

$tmpFile = [System.IO.Path]::GetTempFileName()
write-host $tmpfile

$docheck = $false
$doApply = $false
$doInfo = $false

for ($i=0; $i -lt $args.count; $i++) {
  if ($args[$i] -eq "-check") { $docheck = $true }
  if ($args[$i] -eq "-apply") { $doApply = $true }
  if ($args[$i] -eq "-info") { $doInfo = $true }
}

if ($doapply -eq $true) {
	$docheck = $true
}
	
Function CheckWSMan([String] $computername)
{
  try {
    Invoke-Command -computername $computername -scriptblock { "Hello from $env:computername.$env:userdnsdomain"; $true }
	$true
  } catch [Exception] {
	write-host $_.Exception.ToString()
    $false
  }
};

Function CheckWMIQuery
{
	Param(
		[string]$computername=$env:computername,
		[system.management.automation.PSCredential] $credential = $null
	)

	try {
		$query = "SELECT LogfileName, NumberOfRecords FROM Win32_NTEventlogFile"
		$queryResult = get-wmiobject -computername $computername -credential $credentials -query $query
		$result = ($queryResult -ne $null) -and ($queryResult.length -ge 0)
		$result
	} catch [Exception] {
		Write-Host "Failed remote WMI query..." -foreground "red"
		Write-Host $_.Exception.TOString()
		$false
	}
}

Function ExecuteSubscript(
	[string]$computername,
	[System.Management.Automation.Runspaces.PSSession]$sess, 
	[string]$filePath,
	[string]$message, 
	[boolean]$check
)
{
	$resultOutput = Invoke-Command -session $sess -filePath $filePath
	$text = "$message on $computername ..."

	try
	{
		$logDetails = ""
		for ($i=0; $i -le $resultOutput.length; $i++)
		{
			$logDetails = $logDetails + "`r`n" + $resultOutput[$i]
		}
		$logDetails >> $tmpFile
		
		$result = Invoke-Command -session $sess -scriptblock { $result }

		if ($result.gettype().toString() -eq "System.Object[]") {
			$result = $result[$result.length-1]
		}
		
		try {
			if ($result -eq $true) {
				if ($check -eq $true) {
					write-host "$text ALREADY CONFIGURED" -foreground "green"
				} else {
					write-host "$text SUCCEEDED" -foreground "green"
				}
			} elseif ($result -eq $false) {
				if ($check -eq $true) {
					write-host "$text NOT CONFIGURED" -foreground "red"
				} else {
					write-host "$text FAILED" -foreground "red"
				}
			} else {
				write-host "$text ERROR" -foreground "red"
				write-host $result
				$result = $false
			}
		} catch [Exception] {
			write-host "$text ERROR" -foreground "red"
			write-host $result
			write-host $_.Exception.ToString()
			$result = $false
		}
		
	}
	catch [Exception]
	{
		write-host "$text EXCEPTION" -foreground "red"
	}
	
	return $result
}

if ((-not $docheck -and -not $doApply -and -not $doInfo) -or ($computernames.length -eq 0) -or ($credential -eq $null)) {
  write-host "Sumo-Remote-Collector-Config tool for configuring remote windows event log collection."
  write-host
  write-host "  -check    Checks if remote WMI collection already configured."
  write-host "  -apply    Apply settings to enable remote WMI collection."
  write-host
  write-host "  -computernames [cn1,cn2,cn3,etc]  Computer names separated by commas and no spaces."
  write-host "  -credential [credentials]         Credential object for remote connections (see README.txt for how to create)."
  write-host
  write-host "For help on configuring remote powershell, see README.txt."

  exit  
}

# Automatically enable transcript details logging.
$transcribing = $false
try
{
	$sumoLocation = get-location
	$logFile = $sumoLocation.Path + "\..\logs\psscript-$((get-date).tostring("MMddyyyy.HHmmss")).log"
	start-transcript -path $logFile
	$transcribing = $true
}
catch [Exception]
{
	write-host "Unable to transcribe output to logs." -foreground "red"
}

if ($doInfo -eq $true) {
	write-host ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> " -foreground "cyan"
	write-host "Getting system information on *LOCAL* machine ${currentComputerName} ... (this may take a bit of time)" -foreground "green"
	./scripts/get-sysinfo.ps1 -computername "localhost"
}

foreach ($computername in $computernames) {
        if ($doApply -eq $true) {
		write-host ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> Configuring $computername" -foreground "cyan"
	} elseif ($doCheck -eq $true) {
                write-host ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> Checking $computername" -foreground "cyan"
	} else {
                write-host ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> $computername" -foreground "cyan"
	}

	$isLocal = $computername -ieq "localhost"
	
	if ($isLocal -eq $true) {
	
		$sess = new-pssession -credential $credential
		$result = $false
		
		if ($sess -eq $null)
		{
			write-host "Failed to create local powershell session.  See readme.txt for help." -foreground "red"
			continue
		}
		else
		{
			$result = $true
		}
	} else {
		$sess = new-pssession -computername $computername -credential $credential

		if ($sess -eq $null)
		{
			write-host "Failed to create remote powershell session.  See readme.txt for help." -foreground "red"
			continue
		}

		$result = ExecuteSubscript $computername $sess "./scripts/hello.ps1" "Remote powershell via WSMan" $false
	} 
	write-host ""

	if ($result -eq $true) {

		$resultOutput = Invoke-Command -session $sess -scriptBlock { gwmi -class Win32_OperatingSystem | select Name,OSArchitecture,Caption,CSDVersion,Version }
		write-output $resultOutput
		
		if ($doInfo -eq $true) {
			write-host "Getting system information ... (this may take a bit of time)" -foreground "green"
			Invoke-Command -session $sess -filePath ./scripts/get-sysinfo.ps1
		}
		
		$firewall_already_configured = $false
		$registry_already_configured = $false
		$wmi_already_configured = $false
	
		if ($docheck -eq $true) {
			$firewall_already_configured = ExecuteSubScript $computername $sess "./scripts/check-firewall-settings.ps1" "Checking firewall settings for WMI and RPC" $true 

			$registry_already_configured = ExecuteSubScript $computername $sess "./scripts/check-registry-permissions.ps1" "Checking registry permissions for WBEM Scripting" $true 

			$wmi_aready_configured = ExecuteSubScript $computername $sess "./scripts/check-wmi-service.ps1" "Checking WMI service is running" $true 
		}
	
		if ($doapply -eq $true) {
			if ($firewall_already_configured -ne $true) {
				$ignore_result = ExecuteSubScript $computername $sess "./scripts/change-firewall-settings.ps1" "Changed firewall settings for WMI and RPC" $false
			}
			
			if ($registry_already_configured -ne $true) {
				$ignore_result = ExecuteSubScript $computername $sess "./scripts/change-registry-permissions.ps1" "Changed registry permissions for WBEM Scripting" $false
			}
			
			if ($wmi_already_configured -ne $true) {
				$ignore_result = ExecuteSubScript $computername $sess "./scripts/start-wmi-service.ps1" "Started WMI service" $false
			}
		}
	
	}
	
	remove-pssession $sess
}

if ($transcribing -eq $true) 
{
	stop-transcript
	
	# If there are additional log details, then include them as well.
	try {
	type $tmpFile >> $logFile
	del $tmpFile
	} catch { }
}
